=begin
#ShipEngine API

#ShipEngine's easy-to-use REST API lets you manage all of your shipping needs without worrying about the complexities of different carrier APIs and protocols. We handle all the heavy lifting so you can focus on providing a first-class shipping experience for your customers at the best possible prices.  Each of ShipEngine's features can be used by itself or in conjunction with each other to build powerful shipping functionality into your application or service.  ## Getting Started If you're new to REST APIs then be sure to read our [introduction to REST](https://www.shipengine.com/docs/rest/) to understand the basics. Learn how to [authenticate yourself to ShipEngine](https://www.shipengine.com/docs/auth/), and then use our [sandbox environment](https://www.shipengine.com/docs/sandbox/) to kick the tires and get familiar with our API. If you run into any problems, then be sure to check the [error handling guide](https://www.shipengine.com/docs/errors/) for tips.  Here are some step-by-step **tutorials** to get you started:    - [Learn how to create your first shipping label](https://www.shipengine.com/docs/labels/create-a-label/)   - [Calculate shipping costs and compare rates across carriers](https://www.shipengine.com/docs/rates/)   - [Track packages on-demand or in real time](https://www.shipengine.com/docs/tracking/)   - [Validate mailing addresses anywhere on Earth](https://www.shipengine.com/docs/addresses/validation/)   ## Shipping Labels for Every Major Carrier ShipEngine makes it easy to [create shipping labels for any carrier](https://www.shipengine.com/docs/labels/create-a-label/) and [download them](https://www.shipengine.com/docs/labels/downloading/) in a [variety of file formats](https://www.shipengine.com/docs/labels/formats/). You can even customize labels with your own [messages](https://www.shipengine.com/docs/labels/messages/) and [images](https://www.shipengine.com/docs/labels/branding/).   ## Real-Time Package Tracking With ShipEngine you can [get the current status of a package](https://www.shipengine.com/docs/tracking/) or [subscribe to real-time tracking updates](https://www.shipengine.com/docs/tracking/webhooks/) via webhooks. You can also create [custimized tracking pages](https://www.shipengine.com/docs/tracking/branded-tracking-page/) with your own branding so your customers will always know where their package is.   ## Compare Shipping Costs Across Carriers Make sure you ship as cost-effectively as possible by [comparing rates across carriers](https://www.shipengine.com/docs/rates/get-shipment-rates/) using the ShipEngine Rates API. Or if you don't know the full shipment details yet, then you can [get rate estimates](https://www.shipengine.com/docs/rates/estimate/) with limited address info.   ## Worldwide Address Validation ShipEngine supports [address validation](https://www.shipengine.com/docs/addresses/validation/) for virtually [every country on Earth](https://www.shipengine.com/docs/addresses/validation/countries/), including the United States, Canada, Great Britain, Australia, Germany, France, Norway, Spain, Sweden, Israel, Italy, and over 160 others. 

The version of the OpenAPI document: 1.1.202006302006
Contact: sales@shipengine.com
Generated by: https://openapi-generator.tech
OpenAPI Generator version: 4.3.1

=end

require 'cgi'

module OpenapiClient
  class CarrierAccountsApi
    attr_accessor :api_client

    def initialize(api_client = ApiClient.default)
      @api_client = api_client
    end
    # Connect a carrier account
    # Connect a carrier account
    # @param carrier_name [CarrierName] The carrier name, such as &#x60;stamps_com&#x60;, &#x60;ups&#x60;, &#x60;fedex&#x60;, or &#x60;dhl_express&#x60;.
    # @param connect_carrier_request_body [ConnectCarrierRequestBody] 
    # @param [Hash] opts the optional parameters
    # @return [ConnectCarrierResponseBody]
    def connect_carrier(carrier_name, connect_carrier_request_body, opts = {})
      data, _status_code, _headers = connect_carrier_with_http_info(carrier_name, connect_carrier_request_body, opts)
      data
    end

    # Connect a carrier account
    # Connect a carrier account
    # @param carrier_name [CarrierName] The carrier name, such as &#x60;stamps_com&#x60;, &#x60;ups&#x60;, &#x60;fedex&#x60;, or &#x60;dhl_express&#x60;.
    # @param connect_carrier_request_body [ConnectCarrierRequestBody] 
    # @param [Hash] opts the optional parameters
    # @return [Array<(ConnectCarrierResponseBody, Integer, Hash)>] ConnectCarrierResponseBody data, response status code and response headers
    def connect_carrier_with_http_info(carrier_name, connect_carrier_request_body, opts = {})
      if @api_client.config.debugging
        @api_client.config.logger.debug 'Calling API: CarrierAccountsApi.connect_carrier ...'
      end
      # verify the required parameter 'carrier_name' is set
      if @api_client.config.client_side_validation && carrier_name.nil?
        fail ArgumentError, "Missing the required parameter 'carrier_name' when calling CarrierAccountsApi.connect_carrier"
      end
      # verify the required parameter 'connect_carrier_request_body' is set
      if @api_client.config.client_side_validation && connect_carrier_request_body.nil?
        fail ArgumentError, "Missing the required parameter 'connect_carrier_request_body' when calling CarrierAccountsApi.connect_carrier"
      end
      # resource path
      local_var_path = '/v1/connections/carriers/{carrier_name}'.sub('{' + 'carrier_name' + '}', CGI.escape(carrier_name.to_s))

      # query parameters
      query_params = opts[:query_params] || {}

      # header parameters
      header_params = opts[:header_params] || {}
      # HTTP header 'Accept' (if needed)
      header_params['Accept'] = @api_client.select_header_accept(['application/json'])
      # HTTP header 'Content-Type'
      header_params['Content-Type'] = @api_client.select_header_content_type(['application/json'])

      # form parameters
      form_params = opts[:form_params] || {}

      # http body (model)
      post_body = opts[:body] || @api_client.object_to_http_body(connect_carrier_request_body) 

      # return_type
      return_type = opts[:return_type] || 'ConnectCarrierResponseBody' 

      # auth_names
      auth_names = opts[:auth_names] || ['api_key']

      new_options = opts.merge(
        :header_params => header_params,
        :query_params => query_params,
        :form_params => form_params,
        :body => post_body,
        :auth_names => auth_names,
        :return_type => return_type
      )

      data, status_code, headers = @api_client.call_api(:POST, local_var_path, new_options)
      if @api_client.config.debugging
        @api_client.config.logger.debug "API called: CarrierAccountsApi#connect_carrier\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
      end
      return data, status_code, headers
    end

    # Disconnect a carrier
    # Disconnect a carrier
    # @param carrier_name [CarrierName] The carrier name, such as &#x60;stamps_com&#x60;, &#x60;ups&#x60;, &#x60;fedex&#x60;, or &#x60;dhl_express&#x60;.
    # @param carrier_id [String] Carrier ID
    # @param [Hash] opts the optional parameters
    # @return [String]
    def disconnect_carrier(carrier_name, carrier_id, opts = {})
      data, _status_code, _headers = disconnect_carrier_with_http_info(carrier_name, carrier_id, opts)
      data
    end

    # Disconnect a carrier
    # Disconnect a carrier
    # @param carrier_name [CarrierName] The carrier name, such as &#x60;stamps_com&#x60;, &#x60;ups&#x60;, &#x60;fedex&#x60;, or &#x60;dhl_express&#x60;.
    # @param carrier_id [String] Carrier ID
    # @param [Hash] opts the optional parameters
    # @return [Array<(String, Integer, Hash)>] String data, response status code and response headers
    def disconnect_carrier_with_http_info(carrier_name, carrier_id, opts = {})
      if @api_client.config.debugging
        @api_client.config.logger.debug 'Calling API: CarrierAccountsApi.disconnect_carrier ...'
      end
      # verify the required parameter 'carrier_name' is set
      if @api_client.config.client_side_validation && carrier_name.nil?
        fail ArgumentError, "Missing the required parameter 'carrier_name' when calling CarrierAccountsApi.disconnect_carrier"
      end
      # verify the required parameter 'carrier_id' is set
      if @api_client.config.client_side_validation && carrier_id.nil?
        fail ArgumentError, "Missing the required parameter 'carrier_id' when calling CarrierAccountsApi.disconnect_carrier"
      end
      if @api_client.config.client_side_validation && carrier_id.to_s.length > 25
        fail ArgumentError, 'invalid value for "carrier_id" when calling CarrierAccountsApi.disconnect_carrier, the character length must be smaller than or equal to 25.'
      end

      if @api_client.config.client_side_validation && carrier_id.to_s.length < 1
        fail ArgumentError, 'invalid value for "carrier_id" when calling CarrierAccountsApi.disconnect_carrier, the character length must be great than or equal to 1.'
      end

      pattern = Regexp.new(/^se(-[a-z0-9]+)+$/)
      if @api_client.config.client_side_validation && carrier_id !~ pattern
        fail ArgumentError, "invalid value for 'carrier_id' when calling CarrierAccountsApi.disconnect_carrier, must conform to the pattern #{pattern}."
      end

      # resource path
      local_var_path = '/v1/connections/carriers/{carrier_name}/{carrier_id}'.sub('{' + 'carrier_name' + '}', CGI.escape(carrier_name.to_s)).sub('{' + 'carrier_id' + '}', CGI.escape(carrier_id.to_s))

      # query parameters
      query_params = opts[:query_params] || {}

      # header parameters
      header_params = opts[:header_params] || {}
      # HTTP header 'Accept' (if needed)
      header_params['Accept'] = @api_client.select_header_accept(['text/plain', 'application/json'])

      # form parameters
      form_params = opts[:form_params] || {}

      # http body (model)
      post_body = opts[:body] 

      # return_type
      return_type = opts[:return_type] || 'String' 

      # auth_names
      auth_names = opts[:auth_names] || ['api_key']

      new_options = opts.merge(
        :header_params => header_params,
        :query_params => query_params,
        :form_params => form_params,
        :body => post_body,
        :auth_names => auth_names,
        :return_type => return_type
      )

      data, status_code, headers = @api_client.call_api(:DELETE, local_var_path, new_options)
      if @api_client.config.debugging
        @api_client.config.logger.debug "API called: CarrierAccountsApi#disconnect_carrier\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
      end
      return data, status_code, headers
    end

    # Get carrier settings
    # Get carrier settings
    # @param carrier_name [CarrierNameWithSettings] The carrier name, such as &#x60;stamps_com&#x60;, &#x60;ups&#x60;, &#x60;fedex&#x60;, or &#x60;dhl_express&#x60;.
    # @param carrier_id [String] Carrier ID
    # @param [Hash] opts the optional parameters
    # @return [GetCarrierSettingsResponseBody]
    def get_carrier_settings(carrier_name, carrier_id, opts = {})
      data, _status_code, _headers = get_carrier_settings_with_http_info(carrier_name, carrier_id, opts)
      data
    end

    # Get carrier settings
    # Get carrier settings
    # @param carrier_name [CarrierNameWithSettings] The carrier name, such as &#x60;stamps_com&#x60;, &#x60;ups&#x60;, &#x60;fedex&#x60;, or &#x60;dhl_express&#x60;.
    # @param carrier_id [String] Carrier ID
    # @param [Hash] opts the optional parameters
    # @return [Array<(GetCarrierSettingsResponseBody, Integer, Hash)>] GetCarrierSettingsResponseBody data, response status code and response headers
    def get_carrier_settings_with_http_info(carrier_name, carrier_id, opts = {})
      if @api_client.config.debugging
        @api_client.config.logger.debug 'Calling API: CarrierAccountsApi.get_carrier_settings ...'
      end
      # verify the required parameter 'carrier_name' is set
      if @api_client.config.client_side_validation && carrier_name.nil?
        fail ArgumentError, "Missing the required parameter 'carrier_name' when calling CarrierAccountsApi.get_carrier_settings"
      end
      # verify the required parameter 'carrier_id' is set
      if @api_client.config.client_side_validation && carrier_id.nil?
        fail ArgumentError, "Missing the required parameter 'carrier_id' when calling CarrierAccountsApi.get_carrier_settings"
      end
      if @api_client.config.client_side_validation && carrier_id.to_s.length > 25
        fail ArgumentError, 'invalid value for "carrier_id" when calling CarrierAccountsApi.get_carrier_settings, the character length must be smaller than or equal to 25.'
      end

      if @api_client.config.client_side_validation && carrier_id.to_s.length < 1
        fail ArgumentError, 'invalid value for "carrier_id" when calling CarrierAccountsApi.get_carrier_settings, the character length must be great than or equal to 1.'
      end

      pattern = Regexp.new(/^se(-[a-z0-9]+)+$/)
      if @api_client.config.client_side_validation && carrier_id !~ pattern
        fail ArgumentError, "invalid value for 'carrier_id' when calling CarrierAccountsApi.get_carrier_settings, must conform to the pattern #{pattern}."
      end

      # resource path
      local_var_path = '/v1/connections/carriers/{carrier_name}/{carrier_id}/settings'.sub('{' + 'carrier_name' + '}', CGI.escape(carrier_name.to_s)).sub('{' + 'carrier_id' + '}', CGI.escape(carrier_id.to_s))

      # query parameters
      query_params = opts[:query_params] || {}

      # header parameters
      header_params = opts[:header_params] || {}
      # HTTP header 'Accept' (if needed)
      header_params['Accept'] = @api_client.select_header_accept(['application/json'])

      # form parameters
      form_params = opts[:form_params] || {}

      # http body (model)
      post_body = opts[:body] 

      # return_type
      return_type = opts[:return_type] || 'GetCarrierSettingsResponseBody' 

      # auth_names
      auth_names = opts[:auth_names] || ['api_key']

      new_options = opts.merge(
        :header_params => header_params,
        :query_params => query_params,
        :form_params => form_params,
        :body => post_body,
        :auth_names => auth_names,
        :return_type => return_type
      )

      data, status_code, headers = @api_client.call_api(:GET, local_var_path, new_options)
      if @api_client.config.debugging
        @api_client.config.logger.debug "API called: CarrierAccountsApi#get_carrier_settings\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
      end
      return data, status_code, headers
    end

    # Update carrier settings
    # Update carrier settings
    # @param carrier_name [CarrierNameWithSettings] The carrier name, such as &#x60;stamps_com&#x60;, &#x60;ups&#x60;, &#x60;fedex&#x60;, or &#x60;dhl_express&#x60;.
    # @param carrier_id [String] Carrier ID
    # @param update_carrier_settings_request_body [UpdateCarrierSettingsRequestBody] 
    # @param [Hash] opts the optional parameters
    # @return [String]
    def update_carrier_settings(carrier_name, carrier_id, update_carrier_settings_request_body, opts = {})
      data, _status_code, _headers = update_carrier_settings_with_http_info(carrier_name, carrier_id, update_carrier_settings_request_body, opts)
      data
    end

    # Update carrier settings
    # Update carrier settings
    # @param carrier_name [CarrierNameWithSettings] The carrier name, such as &#x60;stamps_com&#x60;, &#x60;ups&#x60;, &#x60;fedex&#x60;, or &#x60;dhl_express&#x60;.
    # @param carrier_id [String] Carrier ID
    # @param update_carrier_settings_request_body [UpdateCarrierSettingsRequestBody] 
    # @param [Hash] opts the optional parameters
    # @return [Array<(String, Integer, Hash)>] String data, response status code and response headers
    def update_carrier_settings_with_http_info(carrier_name, carrier_id, update_carrier_settings_request_body, opts = {})
      if @api_client.config.debugging
        @api_client.config.logger.debug 'Calling API: CarrierAccountsApi.update_carrier_settings ...'
      end
      # verify the required parameter 'carrier_name' is set
      if @api_client.config.client_side_validation && carrier_name.nil?
        fail ArgumentError, "Missing the required parameter 'carrier_name' when calling CarrierAccountsApi.update_carrier_settings"
      end
      # verify the required parameter 'carrier_id' is set
      if @api_client.config.client_side_validation && carrier_id.nil?
        fail ArgumentError, "Missing the required parameter 'carrier_id' when calling CarrierAccountsApi.update_carrier_settings"
      end
      if @api_client.config.client_side_validation && carrier_id.to_s.length > 25
        fail ArgumentError, 'invalid value for "carrier_id" when calling CarrierAccountsApi.update_carrier_settings, the character length must be smaller than or equal to 25.'
      end

      if @api_client.config.client_side_validation && carrier_id.to_s.length < 1
        fail ArgumentError, 'invalid value for "carrier_id" when calling CarrierAccountsApi.update_carrier_settings, the character length must be great than or equal to 1.'
      end

      pattern = Regexp.new(/^se(-[a-z0-9]+)+$/)
      if @api_client.config.client_side_validation && carrier_id !~ pattern
        fail ArgumentError, "invalid value for 'carrier_id' when calling CarrierAccountsApi.update_carrier_settings, must conform to the pattern #{pattern}."
      end

      # verify the required parameter 'update_carrier_settings_request_body' is set
      if @api_client.config.client_side_validation && update_carrier_settings_request_body.nil?
        fail ArgumentError, "Missing the required parameter 'update_carrier_settings_request_body' when calling CarrierAccountsApi.update_carrier_settings"
      end
      # resource path
      local_var_path = '/v1/connections/carriers/{carrier_name}/{carrier_id}/settings'.sub('{' + 'carrier_name' + '}', CGI.escape(carrier_name.to_s)).sub('{' + 'carrier_id' + '}', CGI.escape(carrier_id.to_s))

      # query parameters
      query_params = opts[:query_params] || {}

      # header parameters
      header_params = opts[:header_params] || {}
      # HTTP header 'Accept' (if needed)
      header_params['Accept'] = @api_client.select_header_accept(['text/plain', 'application/json'])
      # HTTP header 'Content-Type'
      header_params['Content-Type'] = @api_client.select_header_content_type(['application/json'])

      # form parameters
      form_params = opts[:form_params] || {}

      # http body (model)
      post_body = opts[:body] || @api_client.object_to_http_body(update_carrier_settings_request_body) 

      # return_type
      return_type = opts[:return_type] || 'String' 

      # auth_names
      auth_names = opts[:auth_names] || ['api_key']

      new_options = opts.merge(
        :header_params => header_params,
        :query_params => query_params,
        :form_params => form_params,
        :body => post_body,
        :auth_names => auth_names,
        :return_type => return_type
      )

      data, status_code, headers = @api_client.call_api(:PUT, local_var_path, new_options)
      if @api_client.config.debugging
        @api_client.config.logger.debug "API called: CarrierAccountsApi#update_carrier_settings\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
      end
      return data, status_code, headers
    end
  end
end
