=begin
#ShipEngine API

#ShipEngine's easy-to-use REST API lets you manage all of your shipping needs without worrying about the complexities of different carrier APIs and protocols. We handle all the heavy lifting so you can focus on providing a first-class shipping experience for your customers at the best possible prices.  Each of ShipEngine's features can be used by itself or in conjunction with each other to build powerful shipping functionality into your application or service.  ## Getting Started If you're new to REST APIs then be sure to read our [introduction to REST](https://www.shipengine.com/docs/rest/) to understand the basics. Learn how to [authenticate yourself to ShipEngine](https://www.shipengine.com/docs/auth/), and then use our [sandbox environment](https://www.shipengine.com/docs/sandbox/) to kick the tires and get familiar with our API. If you run into any problems, then be sure to check the [error handling guide](https://www.shipengine.com/docs/errors/) for tips.  Here are some step-by-step **tutorials** to get you started:    - [Learn how to create your first shipping label](https://www.shipengine.com/docs/labels/create-a-label/)   - [Calculate shipping costs and compare rates across carriers](https://www.shipengine.com/docs/rates/)   - [Track packages on-demand or in real time](https://www.shipengine.com/docs/tracking/)   - [Validate mailing addresses anywhere on Earth](https://www.shipengine.com/docs/addresses/validation/)   ## Shipping Labels for Every Major Carrier ShipEngine makes it easy to [create shipping labels for any carrier](https://www.shipengine.com/docs/labels/create-a-label/) and [download them](https://www.shipengine.com/docs/labels/downloading/) in a [variety of file formats](https://www.shipengine.com/docs/labels/formats/). You can even customize labels with your own [messages](https://www.shipengine.com/docs/labels/messages/) and [images](https://www.shipengine.com/docs/labels/branding/).   ## Real-Time Package Tracking With ShipEngine you can [get the current status of a package](https://www.shipengine.com/docs/tracking/) or [subscribe to real-time tracking updates](https://www.shipengine.com/docs/tracking/webhooks/) via webhooks. You can also create [custimized tracking pages](https://www.shipengine.com/docs/tracking/branded-tracking-page/) with your own branding so your customers will always know where their package is.   ## Compare Shipping Costs Across Carriers Make sure you ship as cost-effectively as possible by [comparing rates across carriers](https://www.shipengine.com/docs/rates/get-shipment-rates/) using the ShipEngine Rates API. Or if you don't know the full shipment details yet, then you can [get rate estimates](https://www.shipengine.com/docs/rates/estimate/) with limited address info.   ## Worldwide Address Validation ShipEngine supports [address validation](https://www.shipengine.com/docs/addresses/validation/) for virtually [every country on Earth](https://www.shipengine.com/docs/addresses/validation/countries/), including the United States, Canada, Great Britain, Australia, Germany, France, Norway, Spain, Sweden, Israel, Italy, and over 160 others. 

The version of the OpenAPI document: 1.1.202006302006
Contact: sales@shipengine.com
Generated by: https://openapi-generator.tech
OpenAPI Generator version: 4.3.1

=end

require 'cgi'

module OpenapiClient
  class ManifestsApi
    attr_accessor :api_client

    def initialize(api_client = ApiClient.default)
      @api_client = api_client
    end
    # Create Manifest
    # Each ShipEngine manifest is created for a specific warehouse, so you'll need to provide the warehouse_id rather than the ship_from address. You can create a warehouse for each location that you want to create manifests for. 
    # @param create_manifest_request_body [CreateManifestRequestBody] 
    # @param [Hash] opts the optional parameters
    # @return [CreateManifestResponseBody]
    def create_manifest(create_manifest_request_body, opts = {})
      data, _status_code, _headers = create_manifest_with_http_info(create_manifest_request_body, opts)
      data
    end

    # Create Manifest
    # Each ShipEngine manifest is created for a specific warehouse, so you&#39;ll need to provide the warehouse_id rather than the ship_from address. You can create a warehouse for each location that you want to create manifests for. 
    # @param create_manifest_request_body [CreateManifestRequestBody] 
    # @param [Hash] opts the optional parameters
    # @return [Array<(CreateManifestResponseBody, Integer, Hash)>] CreateManifestResponseBody data, response status code and response headers
    def create_manifest_with_http_info(create_manifest_request_body, opts = {})
      if @api_client.config.debugging
        @api_client.config.logger.debug 'Calling API: ManifestsApi.create_manifest ...'
      end
      # verify the required parameter 'create_manifest_request_body' is set
      if @api_client.config.client_side_validation && create_manifest_request_body.nil?
        fail ArgumentError, "Missing the required parameter 'create_manifest_request_body' when calling ManifestsApi.create_manifest"
      end
      # resource path
      local_var_path = '/v1/manifests'

      # query parameters
      query_params = opts[:query_params] || {}

      # header parameters
      header_params = opts[:header_params] || {}
      # HTTP header 'Accept' (if needed)
      header_params['Accept'] = @api_client.select_header_accept(['application/json'])
      # HTTP header 'Content-Type'
      header_params['Content-Type'] = @api_client.select_header_content_type(['application/json'])

      # form parameters
      form_params = opts[:form_params] || {}

      # http body (model)
      post_body = opts[:body] || @api_client.object_to_http_body(create_manifest_request_body) 

      # return_type
      return_type = opts[:return_type] || 'CreateManifestResponseBody' 

      # auth_names
      auth_names = opts[:auth_names] || ['api_key']

      new_options = opts.merge(
        :header_params => header_params,
        :query_params => query_params,
        :form_params => form_params,
        :body => post_body,
        :auth_names => auth_names,
        :return_type => return_type
      )

      data, status_code, headers = @api_client.call_api(:POST, local_var_path, new_options)
      if @api_client.config.debugging
        @api_client.config.logger.debug "API called: ManifestsApi#create_manifest\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
      end
      return data, status_code, headers
    end

    # Get Manifest By Id
    # Get Manifest By Id
    # @param manifest_id [String] The Manifest Id
    # @param [Hash] opts the optional parameters
    # @return [GetManifestByIdResponseBody]
    def get_manifest_by_id(manifest_id, opts = {})
      data, _status_code, _headers = get_manifest_by_id_with_http_info(manifest_id, opts)
      data
    end

    # Get Manifest By Id
    # Get Manifest By Id
    # @param manifest_id [String] The Manifest Id
    # @param [Hash] opts the optional parameters
    # @return [Array<(GetManifestByIdResponseBody, Integer, Hash)>] GetManifestByIdResponseBody data, response status code and response headers
    def get_manifest_by_id_with_http_info(manifest_id, opts = {})
      if @api_client.config.debugging
        @api_client.config.logger.debug 'Calling API: ManifestsApi.get_manifest_by_id ...'
      end
      # verify the required parameter 'manifest_id' is set
      if @api_client.config.client_side_validation && manifest_id.nil?
        fail ArgumentError, "Missing the required parameter 'manifest_id' when calling ManifestsApi.get_manifest_by_id"
      end
      if @api_client.config.client_side_validation && manifest_id.to_s.length > 25
        fail ArgumentError, 'invalid value for "manifest_id" when calling ManifestsApi.get_manifest_by_id, the character length must be smaller than or equal to 25.'
      end

      if @api_client.config.client_side_validation && manifest_id.to_s.length < 1
        fail ArgumentError, 'invalid value for "manifest_id" when calling ManifestsApi.get_manifest_by_id, the character length must be great than or equal to 1.'
      end

      pattern = Regexp.new(/^se(-[a-z0-9]+)+$/)
      if @api_client.config.client_side_validation && manifest_id !~ pattern
        fail ArgumentError, "invalid value for 'manifest_id' when calling ManifestsApi.get_manifest_by_id, must conform to the pattern #{pattern}."
      end

      # resource path
      local_var_path = '/v1/manifests/{manifest_id}'.sub('{' + 'manifest_id' + '}', CGI.escape(manifest_id.to_s))

      # query parameters
      query_params = opts[:query_params] || {}

      # header parameters
      header_params = opts[:header_params] || {}
      # HTTP header 'Accept' (if needed)
      header_params['Accept'] = @api_client.select_header_accept(['application/json'])

      # form parameters
      form_params = opts[:form_params] || {}

      # http body (model)
      post_body = opts[:body] 

      # return_type
      return_type = opts[:return_type] || 'GetManifestByIdResponseBody' 

      # auth_names
      auth_names = opts[:auth_names] || ['api_key']

      new_options = opts.merge(
        :header_params => header_params,
        :query_params => query_params,
        :form_params => form_params,
        :body => post_body,
        :auth_names => auth_names,
        :return_type => return_type
      )

      data, status_code, headers = @api_client.call_api(:GET, local_var_path, new_options)
      if @api_client.config.debugging
        @api_client.config.logger.debug "API called: ManifestsApi#get_manifest_by_id\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
      end
      return data, status_code, headers
    end

    # List Manifests
    # Similar to querying shipments, we allow you to query manifests since there will likely be a large number over a long period of time.
    # @param [Hash] opts the optional parameters
    # @option opts [String] :warehouse_id Warehouse ID
    # @option opts [DateTime] :ship_date_start ship date start range
    # @option opts [DateTime] :ship_date_end ship date end range
    # @option opts [DateTime] :created_at_start Used to create a filter for when a resource was created (ex. A shipment that was created after a certain time)
    # @option opts [DateTime] :created_at_end Used to create a filter for when a resource was created, (ex. A shipment that was created before a certain time)
    # @option opts [String] :carrier_id Carrier ID
    # @option opts [Integer] :page Return a specific page of results. Defaults to the first page. If set to a number that&#39;s greater than the number of pages of results, an empty page is returned.  (default to 1)
    # @option opts [Integer] :page_size The number of results to return per response. (default to 25)
    # @option opts [Array<String>] :label_ids 
    # @return [ListManifestsResponseBody]
    def list_manifests(opts = {})
      data, _status_code, _headers = list_manifests_with_http_info(opts)
      data
    end

    # List Manifests
    # Similar to querying shipments, we allow you to query manifests since there will likely be a large number over a long period of time.
    # @param [Hash] opts the optional parameters
    # @option opts [String] :warehouse_id Warehouse ID
    # @option opts [DateTime] :ship_date_start ship date start range
    # @option opts [DateTime] :ship_date_end ship date end range
    # @option opts [DateTime] :created_at_start Used to create a filter for when a resource was created (ex. A shipment that was created after a certain time)
    # @option opts [DateTime] :created_at_end Used to create a filter for when a resource was created, (ex. A shipment that was created before a certain time)
    # @option opts [String] :carrier_id Carrier ID
    # @option opts [Integer] :page Return a specific page of results. Defaults to the first page. If set to a number that&#39;s greater than the number of pages of results, an empty page is returned. 
    # @option opts [Integer] :page_size The number of results to return per response.
    # @option opts [Array<String>] :label_ids 
    # @return [Array<(ListManifestsResponseBody, Integer, Hash)>] ListManifestsResponseBody data, response status code and response headers
    def list_manifests_with_http_info(opts = {})
      if @api_client.config.debugging
        @api_client.config.logger.debug 'Calling API: ManifestsApi.list_manifests ...'
      end
      if @api_client.config.client_side_validation && !opts[:'warehouse_id'].nil? && opts[:'warehouse_id'].to_s.length > 25
        fail ArgumentError, 'invalid value for "opts[:"warehouse_id"]" when calling ManifestsApi.list_manifests, the character length must be smaller than or equal to 25.'
      end

      if @api_client.config.client_side_validation && !opts[:'warehouse_id'].nil? && opts[:'warehouse_id'].to_s.length < 1
        fail ArgumentError, 'invalid value for "opts[:"warehouse_id"]" when calling ManifestsApi.list_manifests, the character length must be great than or equal to 1.'
      end

      pattern = Regexp.new(/^se(-[a-z0-9]+)+$/)
      if @api_client.config.client_side_validation && !opts[:'warehouse_id'].nil? && opts[:'warehouse_id'] !~ pattern
        fail ArgumentError, "invalid value for 'opts[:\"warehouse_id\"]' when calling ManifestsApi.list_manifests, must conform to the pattern #{pattern}."
      end

      if @api_client.config.client_side_validation && !opts[:'carrier_id'].nil? && opts[:'carrier_id'].to_s.length > 25
        fail ArgumentError, 'invalid value for "opts[:"carrier_id"]" when calling ManifestsApi.list_manifests, the character length must be smaller than or equal to 25.'
      end

      if @api_client.config.client_side_validation && !opts[:'carrier_id'].nil? && opts[:'carrier_id'].to_s.length < 1
        fail ArgumentError, 'invalid value for "opts[:"carrier_id"]" when calling ManifestsApi.list_manifests, the character length must be great than or equal to 1.'
      end

      pattern = Regexp.new(/^se(-[a-z0-9]+)+$/)
      if @api_client.config.client_side_validation && !opts[:'carrier_id'].nil? && opts[:'carrier_id'] !~ pattern
        fail ArgumentError, "invalid value for 'opts[:\"carrier_id\"]' when calling ManifestsApi.list_manifests, must conform to the pattern #{pattern}."
      end

      if @api_client.config.client_side_validation && !opts[:'page'].nil? && opts[:'page'] < 1
        fail ArgumentError, 'invalid value for "opts[:"page"]" when calling ManifestsApi.list_manifests, must be greater than or equal to 1.'
      end

      if @api_client.config.client_side_validation && !opts[:'page_size'].nil? && opts[:'page_size'] < 1
        fail ArgumentError, 'invalid value for "opts[:"page_size"]" when calling ManifestsApi.list_manifests, must be greater than or equal to 1.'
      end

      # resource path
      local_var_path = '/v1/manifests'

      # query parameters
      query_params = opts[:query_params] || {}
      query_params[:'warehouse_id'] = opts[:'warehouse_id'] if !opts[:'warehouse_id'].nil?
      query_params[:'ship_date_start'] = opts[:'ship_date_start'] if !opts[:'ship_date_start'].nil?
      query_params[:'ship_date_end'] = opts[:'ship_date_end'] if !opts[:'ship_date_end'].nil?
      query_params[:'created_at_start'] = opts[:'created_at_start'] if !opts[:'created_at_start'].nil?
      query_params[:'created_at_end'] = opts[:'created_at_end'] if !opts[:'created_at_end'].nil?
      query_params[:'carrier_id'] = opts[:'carrier_id'] if !opts[:'carrier_id'].nil?
      query_params[:'page'] = opts[:'page'] if !opts[:'page'].nil?
      query_params[:'page_size'] = opts[:'page_size'] if !opts[:'page_size'].nil?
      query_params[:'label_ids'] = @api_client.build_collection_param(opts[:'label_ids'], :multi) if !opts[:'label_ids'].nil?

      # header parameters
      header_params = opts[:header_params] || {}
      # HTTP header 'Accept' (if needed)
      header_params['Accept'] = @api_client.select_header_accept(['application/json'])

      # form parameters
      form_params = opts[:form_params] || {}

      # http body (model)
      post_body = opts[:body] 

      # return_type
      return_type = opts[:return_type] || 'ListManifestsResponseBody' 

      # auth_names
      auth_names = opts[:auth_names] || ['api_key']

      new_options = opts.merge(
        :header_params => header_params,
        :query_params => query_params,
        :form_params => form_params,
        :body => post_body,
        :auth_names => auth_names,
        :return_type => return_type
      )

      data, status_code, headers = @api_client.call_api(:GET, local_var_path, new_options)
      if @api_client.config.debugging
        @api_client.config.logger.debug "API called: ManifestsApi#list_manifests\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
      end
      return data, status_code, headers
    end
  end
end
