=begin
#ShipEngine API

#ShipEngine's easy-to-use REST API lets you manage all of your shipping needs without worrying about the complexities of different carrier APIs and protocols. We handle all the heavy lifting so you can focus on providing a first-class shipping experience for your customers at the best possible prices.  Each of ShipEngine's features can be used by itself or in conjunction with each other to build powerful shipping functionality into your application or service.  ## Getting Started If you're new to REST APIs then be sure to read our [introduction to REST](https://www.shipengine.com/docs/rest/) to understand the basics. Learn how to [authenticate yourself to ShipEngine](https://www.shipengine.com/docs/auth/), and then use our [sandbox environment](https://www.shipengine.com/docs/sandbox/) to kick the tires and get familiar with our API. If you run into any problems, then be sure to check the [error handling guide](https://www.shipengine.com/docs/errors/) for tips.  Here are some step-by-step **tutorials** to get you started:    - [Learn how to create your first shipping label](https://www.shipengine.com/docs/labels/create-a-label/)   - [Calculate shipping costs and compare rates across carriers](https://www.shipengine.com/docs/rates/)   - [Track packages on-demand or in real time](https://www.shipengine.com/docs/tracking/)   - [Validate mailing addresses anywhere on Earth](https://www.shipengine.com/docs/addresses/validation/)   ## Shipping Labels for Every Major Carrier ShipEngine makes it easy to [create shipping labels for any carrier](https://www.shipengine.com/docs/labels/create-a-label/) and [download them](https://www.shipengine.com/docs/labels/downloading/) in a [variety of file formats](https://www.shipengine.com/docs/labels/formats/). You can even customize labels with your own [messages](https://www.shipengine.com/docs/labels/messages/) and [images](https://www.shipengine.com/docs/labels/branding/).   ## Real-Time Package Tracking With ShipEngine you can [get the current status of a package](https://www.shipengine.com/docs/tracking/) or [subscribe to real-time tracking updates](https://www.shipengine.com/docs/tracking/webhooks/) via webhooks. You can also create [custimized tracking pages](https://www.shipengine.com/docs/tracking/branded-tracking-page/) with your own branding so your customers will always know where their package is.   ## Compare Shipping Costs Across Carriers Make sure you ship as cost-effectively as possible by [comparing rates across carriers](https://www.shipengine.com/docs/rates/get-shipment-rates/) using the ShipEngine Rates API. Or if you don't know the full shipment details yet, then you can [get rate estimates](https://www.shipengine.com/docs/rates/estimate/) with limited address info.   ## Worldwide Address Validation ShipEngine supports [address validation](https://www.shipengine.com/docs/addresses/validation/) for virtually [every country on Earth](https://www.shipengine.com/docs/addresses/validation/countries/), including the United States, Canada, Great Britain, Australia, Germany, France, Norway, Spain, Sweden, Israel, Italy, and over 160 others. 

The version of the OpenAPI document: 1.1.202006302006
Contact: sales@shipengine.com
Generated by: https://openapi-generator.tech
OpenAPI Generator version: 4.3.1

=end

require 'cgi'

module OpenapiClient
  class TagsApi
    attr_accessor :api_client

    def initialize(api_client = ApiClient.default)
      @api_client = api_client
    end
    # Create a New Tag
    # Create a new Tag for customizing how you track your shipments
    # @param tag_name [String] 
    # @param [Hash] opts the optional parameters
    # @return [CreateTagResponseBody]
    def create_tag(tag_name, opts = {})
      data, _status_code, _headers = create_tag_with_http_info(tag_name, opts)
      data
    end

    # Create a New Tag
    # Create a new Tag for customizing how you track your shipments
    # @param tag_name [String] 
    # @param [Hash] opts the optional parameters
    # @return [Array<(CreateTagResponseBody, Integer, Hash)>] CreateTagResponseBody data, response status code and response headers
    def create_tag_with_http_info(tag_name, opts = {})
      if @api_client.config.debugging
        @api_client.config.logger.debug 'Calling API: TagsApi.create_tag ...'
      end
      # verify the required parameter 'tag_name' is set
      if @api_client.config.client_side_validation && tag_name.nil?
        fail ArgumentError, "Missing the required parameter 'tag_name' when calling TagsApi.create_tag"
      end
      if @api_client.config.client_side_validation && tag_name.to_s.length < 1
        fail ArgumentError, 'invalid value for "tag_name" when calling TagsApi.create_tag, the character length must be great than or equal to 1.'
      end

      # resource path
      local_var_path = '/v1/tags/{tag_name}'.sub('{' + 'tag_name' + '}', CGI.escape(tag_name.to_s))

      # query parameters
      query_params = opts[:query_params] || {}

      # header parameters
      header_params = opts[:header_params] || {}
      # HTTP header 'Accept' (if needed)
      header_params['Accept'] = @api_client.select_header_accept(['application/json'])

      # form parameters
      form_params = opts[:form_params] || {}

      # http body (model)
      post_body = opts[:body] 

      # return_type
      return_type = opts[:return_type] || 'CreateTagResponseBody' 

      # auth_names
      auth_names = opts[:auth_names] || ['api_key']

      new_options = opts.merge(
        :header_params => header_params,
        :query_params => query_params,
        :form_params => form_params,
        :body => post_body,
        :auth_names => auth_names,
        :return_type => return_type
      )

      data, status_code, headers = @api_client.call_api(:POST, local_var_path, new_options)
      if @api_client.config.debugging
        @api_client.config.logger.debug "API called: TagsApi#create_tag\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
      end
      return data, status_code, headers
    end

    # Delete Tag
    # Delete a tag that is no longer needed
    # @param tag_name [String] 
    # @param [Hash] opts the optional parameters
    # @return [String]
    def delete_tag(tag_name, opts = {})
      data, _status_code, _headers = delete_tag_with_http_info(tag_name, opts)
      data
    end

    # Delete Tag
    # Delete a tag that is no longer needed
    # @param tag_name [String] 
    # @param [Hash] opts the optional parameters
    # @return [Array<(String, Integer, Hash)>] String data, response status code and response headers
    def delete_tag_with_http_info(tag_name, opts = {})
      if @api_client.config.debugging
        @api_client.config.logger.debug 'Calling API: TagsApi.delete_tag ...'
      end
      # verify the required parameter 'tag_name' is set
      if @api_client.config.client_side_validation && tag_name.nil?
        fail ArgumentError, "Missing the required parameter 'tag_name' when calling TagsApi.delete_tag"
      end
      if @api_client.config.client_side_validation && tag_name.to_s.length < 1
        fail ArgumentError, 'invalid value for "tag_name" when calling TagsApi.delete_tag, the character length must be great than or equal to 1.'
      end

      # resource path
      local_var_path = '/v1/tags/{tag_name}'.sub('{' + 'tag_name' + '}', CGI.escape(tag_name.to_s))

      # query parameters
      query_params = opts[:query_params] || {}

      # header parameters
      header_params = opts[:header_params] || {}
      # HTTP header 'Accept' (if needed)
      header_params['Accept'] = @api_client.select_header_accept(['text/plain', 'application/json'])

      # form parameters
      form_params = opts[:form_params] || {}

      # http body (model)
      post_body = opts[:body] 

      # return_type
      return_type = opts[:return_type] || 'String' 

      # auth_names
      auth_names = opts[:auth_names] || ['api_key']

      new_options = opts.merge(
        :header_params => header_params,
        :query_params => query_params,
        :form_params => form_params,
        :body => post_body,
        :auth_names => auth_names,
        :return_type => return_type
      )

      data, status_code, headers = @api_client.call_api(:DELETE, local_var_path, new_options)
      if @api_client.config.debugging
        @api_client.config.logger.debug "API called: TagsApi#delete_tag\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
      end
      return data, status_code, headers
    end

    # Get Tags
    # Get a list of all tags associated with an account.
    # @param [Hash] opts the optional parameters
    # @return [ListTagsResponseBody]
    def list_tags(opts = {})
      data, _status_code, _headers = list_tags_with_http_info(opts)
      data
    end

    # Get Tags
    # Get a list of all tags associated with an account.
    # @param [Hash] opts the optional parameters
    # @return [Array<(ListTagsResponseBody, Integer, Hash)>] ListTagsResponseBody data, response status code and response headers
    def list_tags_with_http_info(opts = {})
      if @api_client.config.debugging
        @api_client.config.logger.debug 'Calling API: TagsApi.list_tags ...'
      end
      # resource path
      local_var_path = '/v1/tags'

      # query parameters
      query_params = opts[:query_params] || {}

      # header parameters
      header_params = opts[:header_params] || {}
      # HTTP header 'Accept' (if needed)
      header_params['Accept'] = @api_client.select_header_accept(['application/json'])

      # form parameters
      form_params = opts[:form_params] || {}

      # http body (model)
      post_body = opts[:body] 

      # return_type
      return_type = opts[:return_type] || 'ListTagsResponseBody' 

      # auth_names
      auth_names = opts[:auth_names] || ['api_key']

      new_options = opts.merge(
        :header_params => header_params,
        :query_params => query_params,
        :form_params => form_params,
        :body => post_body,
        :auth_names => auth_names,
        :return_type => return_type
      )

      data, status_code, headers = @api_client.call_api(:GET, local_var_path, new_options)
      if @api_client.config.debugging
        @api_client.config.logger.debug "API called: TagsApi#list_tags\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
      end
      return data, status_code, headers
    end

    # Update Tag Name
    # Change a tag name while still keeping the relevant shipments attached to it
    # @param tag_name [String] 
    # @param new_tag_name [String] 
    # @param [Hash] opts the optional parameters
    # @return [String]
    def rename_tag(tag_name, new_tag_name, opts = {})
      data, _status_code, _headers = rename_tag_with_http_info(tag_name, new_tag_name, opts)
      data
    end

    # Update Tag Name
    # Change a tag name while still keeping the relevant shipments attached to it
    # @param tag_name [String] 
    # @param new_tag_name [String] 
    # @param [Hash] opts the optional parameters
    # @return [Array<(String, Integer, Hash)>] String data, response status code and response headers
    def rename_tag_with_http_info(tag_name, new_tag_name, opts = {})
      if @api_client.config.debugging
        @api_client.config.logger.debug 'Calling API: TagsApi.rename_tag ...'
      end
      # verify the required parameter 'tag_name' is set
      if @api_client.config.client_side_validation && tag_name.nil?
        fail ArgumentError, "Missing the required parameter 'tag_name' when calling TagsApi.rename_tag"
      end
      if @api_client.config.client_side_validation && tag_name.to_s.length < 1
        fail ArgumentError, 'invalid value for "tag_name" when calling TagsApi.rename_tag, the character length must be great than or equal to 1.'
      end

      # verify the required parameter 'new_tag_name' is set
      if @api_client.config.client_side_validation && new_tag_name.nil?
        fail ArgumentError, "Missing the required parameter 'new_tag_name' when calling TagsApi.rename_tag"
      end
      if @api_client.config.client_side_validation && new_tag_name.to_s.length < 1
        fail ArgumentError, 'invalid value for "new_tag_name" when calling TagsApi.rename_tag, the character length must be great than or equal to 1.'
      end

      # resource path
      local_var_path = '/v1/tags/{tag_name}/{new_tag_name}'.sub('{' + 'tag_name' + '}', CGI.escape(tag_name.to_s)).sub('{' + 'new_tag_name' + '}', CGI.escape(new_tag_name.to_s))

      # query parameters
      query_params = opts[:query_params] || {}

      # header parameters
      header_params = opts[:header_params] || {}
      # HTTP header 'Accept' (if needed)
      header_params['Accept'] = @api_client.select_header_accept(['text/plain', 'application/json'])

      # form parameters
      form_params = opts[:form_params] || {}

      # http body (model)
      post_body = opts[:body] 

      # return_type
      return_type = opts[:return_type] || 'String' 

      # auth_names
      auth_names = opts[:auth_names] || ['api_key']

      new_options = opts.merge(
        :header_params => header_params,
        :query_params => query_params,
        :form_params => form_params,
        :body => post_body,
        :auth_names => auth_names,
        :return_type => return_type
      )

      data, status_code, headers = @api_client.call_api(:PUT, local_var_path, new_options)
      if @api_client.config.debugging
        @api_client.config.logger.debug "API called: TagsApi#rename_tag\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
      end
      return data, status_code, headers
    end
  end
end
