=begin
#ShipEngine API

#ShipEngine's easy-to-use REST API lets you manage all of your shipping needs without worrying about the complexities of different carrier APIs and protocols. We handle all the heavy lifting so you can focus on providing a first-class shipping experience for your customers at the best possible prices.  Each of ShipEngine's features can be used by itself or in conjunction with each other to build powerful shipping functionality into your application or service.  ## Getting Started If you're new to REST APIs then be sure to read our [introduction to REST](https://www.shipengine.com/docs/rest/) to understand the basics. Learn how to [authenticate yourself to ShipEngine](https://www.shipengine.com/docs/auth/), and then use our [sandbox environment](https://www.shipengine.com/docs/sandbox/) to kick the tires and get familiar with our API. If you run into any problems, then be sure to check the [error handling guide](https://www.shipengine.com/docs/errors/) for tips.  Here are some step-by-step **tutorials** to get you started:    - [Learn how to create your first shipping label](https://www.shipengine.com/docs/labels/create-a-label/)   - [Calculate shipping costs and compare rates across carriers](https://www.shipengine.com/docs/rates/)   - [Track packages on-demand or in real time](https://www.shipengine.com/docs/tracking/)   - [Validate mailing addresses anywhere on Earth](https://www.shipengine.com/docs/addresses/validation/)   ## Shipping Labels for Every Major Carrier ShipEngine makes it easy to [create shipping labels for any carrier](https://www.shipengine.com/docs/labels/create-a-label/) and [download them](https://www.shipengine.com/docs/labels/downloading/) in a [variety of file formats](https://www.shipengine.com/docs/labels/formats/). You can even customize labels with your own [messages](https://www.shipengine.com/docs/labels/messages/) and [images](https://www.shipengine.com/docs/labels/branding/).   ## Real-Time Package Tracking With ShipEngine you can [get the current status of a package](https://www.shipengine.com/docs/tracking/) or [subscribe to real-time tracking updates](https://www.shipengine.com/docs/tracking/webhooks/) via webhooks. You can also create [custimized tracking pages](https://www.shipengine.com/docs/tracking/branded-tracking-page/) with your own branding so your customers will always know where their package is.   ## Compare Shipping Costs Across Carriers Make sure you ship as cost-effectively as possible by [comparing rates across carriers](https://www.shipengine.com/docs/rates/get-shipment-rates/) using the ShipEngine Rates API. Or if you don't know the full shipment details yet, then you can [get rate estimates](https://www.shipengine.com/docs/rates/estimate/) with limited address info.   ## Worldwide Address Validation ShipEngine supports [address validation](https://www.shipengine.com/docs/addresses/validation/) for virtually [every country on Earth](https://www.shipengine.com/docs/addresses/validation/countries/), including the United States, Canada, Great Britain, Australia, Germany, France, Norway, Spain, Sweden, Israel, Italy, and over 160 others. 

The version of the OpenAPI document: 1.1.202006302006
Contact: sales@shipengine.com
Generated by: https://openapi-generator.tech
OpenAPI Generator version: 4.3.1

=end

require 'cgi'

module OpenapiClient
  class TrackingApi
    attr_accessor :api_client

    def initialize(api_client = ApiClient.default)
      @api_client = api_client
    end
    # Get Tracking Information
    # Retrieve package tracking information
    # @param [Hash] opts the optional parameters
    # @option opts [String] :carrier_code Carrier code used to retrieve tracking information
    # @option opts [String] :tracking_number The tracking number associated with a shipment
    # @return [GetTrackingLogResponseBody]
    def get_tracking_log(opts = {})
      data, _status_code, _headers = get_tracking_log_with_http_info(opts)
      data
    end

    # Get Tracking Information
    # Retrieve package tracking information
    # @param [Hash] opts the optional parameters
    # @option opts [String] :carrier_code Carrier code used to retrieve tracking information
    # @option opts [String] :tracking_number The tracking number associated with a shipment
    # @return [Array<(GetTrackingLogResponseBody, Integer, Hash)>] GetTrackingLogResponseBody data, response status code and response headers
    def get_tracking_log_with_http_info(opts = {})
      if @api_client.config.debugging
        @api_client.config.logger.debug 'Calling API: TrackingApi.get_tracking_log ...'
      end
      if @api_client.config.client_side_validation && !opts[:'carrier_code'].nil? && opts[:'carrier_code'].to_s.length < 1
        fail ArgumentError, 'invalid value for "opts[:"carrier_code"]" when calling TrackingApi.get_tracking_log, the character length must be great than or equal to 1.'
      end

      if @api_client.config.client_side_validation && !opts[:'tracking_number'].nil? && opts[:'tracking_number'].to_s.length < 1
        fail ArgumentError, 'invalid value for "opts[:"tracking_number"]" when calling TrackingApi.get_tracking_log, the character length must be great than or equal to 1.'
      end

      # resource path
      local_var_path = '/v1/tracking'

      # query parameters
      query_params = opts[:query_params] || {}
      query_params[:'carrier_code'] = opts[:'carrier_code'] if !opts[:'carrier_code'].nil?
      query_params[:'tracking_number'] = opts[:'tracking_number'] if !opts[:'tracking_number'].nil?

      # header parameters
      header_params = opts[:header_params] || {}
      # HTTP header 'Accept' (if needed)
      header_params['Accept'] = @api_client.select_header_accept(['application/json'])

      # form parameters
      form_params = opts[:form_params] || {}

      # http body (model)
      post_body = opts[:body] 

      # return_type
      return_type = opts[:return_type] || 'GetTrackingLogResponseBody' 

      # auth_names
      auth_names = opts[:auth_names] || ['api_key']

      new_options = opts.merge(
        :header_params => header_params,
        :query_params => query_params,
        :form_params => form_params,
        :body => post_body,
        :auth_names => auth_names,
        :return_type => return_type
      )

      data, status_code, headers = @api_client.call_api(:GET, local_var_path, new_options)
      if @api_client.config.debugging
        @api_client.config.logger.debug "API called: TrackingApi#get_tracking_log\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
      end
      return data, status_code, headers
    end

    # Start Tracking a Package
    # Allows you to subscribe to tracking updates for a package. You specify the carrier_code and tracking_number of the package, and receive notifications via webhooks whenever the shipping status changes. 
    # @param [Hash] opts the optional parameters
    # @option opts [String] :carrier_code Carrier code used to retrieve tracking information
    # @option opts [String] :tracking_number The tracking number associated with a shipment
    # @return [String]
    def start_tracking(opts = {})
      data, _status_code, _headers = start_tracking_with_http_info(opts)
      data
    end

    # Start Tracking a Package
    # Allows you to subscribe to tracking updates for a package. You specify the carrier_code and tracking_number of the package, and receive notifications via webhooks whenever the shipping status changes. 
    # @param [Hash] opts the optional parameters
    # @option opts [String] :carrier_code Carrier code used to retrieve tracking information
    # @option opts [String] :tracking_number The tracking number associated with a shipment
    # @return [Array<(String, Integer, Hash)>] String data, response status code and response headers
    def start_tracking_with_http_info(opts = {})
      if @api_client.config.debugging
        @api_client.config.logger.debug 'Calling API: TrackingApi.start_tracking ...'
      end
      if @api_client.config.client_side_validation && !opts[:'carrier_code'].nil? && opts[:'carrier_code'].to_s.length < 1
        fail ArgumentError, 'invalid value for "opts[:"carrier_code"]" when calling TrackingApi.start_tracking, the character length must be great than or equal to 1.'
      end

      if @api_client.config.client_side_validation && !opts[:'tracking_number'].nil? && opts[:'tracking_number'].to_s.length < 1
        fail ArgumentError, 'invalid value for "opts[:"tracking_number"]" when calling TrackingApi.start_tracking, the character length must be great than or equal to 1.'
      end

      # resource path
      local_var_path = '/v1/tracking/start'

      # query parameters
      query_params = opts[:query_params] || {}
      query_params[:'carrier_code'] = opts[:'carrier_code'] if !opts[:'carrier_code'].nil?
      query_params[:'tracking_number'] = opts[:'tracking_number'] if !opts[:'tracking_number'].nil?

      # header parameters
      header_params = opts[:header_params] || {}
      # HTTP header 'Accept' (if needed)
      header_params['Accept'] = @api_client.select_header_accept(['text/plain', 'application/json'])

      # form parameters
      form_params = opts[:form_params] || {}

      # http body (model)
      post_body = opts[:body] 

      # return_type
      return_type = opts[:return_type] || 'String' 

      # auth_names
      auth_names = opts[:auth_names] || ['api_key']

      new_options = opts.merge(
        :header_params => header_params,
        :query_params => query_params,
        :form_params => form_params,
        :body => post_body,
        :auth_names => auth_names,
        :return_type => return_type
      )

      data, status_code, headers = @api_client.call_api(:POST, local_var_path, new_options)
      if @api_client.config.debugging
        @api_client.config.logger.debug "API called: TrackingApi#start_tracking\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
      end
      return data, status_code, headers
    end

    # Stop Tracking a Package
    # Unsubscribe from tracking updates for a package.
    # @param [Hash] opts the optional parameters
    # @option opts [String] :carrier_code Carrier code used to retrieve tracking information
    # @option opts [String] :tracking_number The tracking number associated with a shipment
    # @return [String]
    def stop_tracking(opts = {})
      data, _status_code, _headers = stop_tracking_with_http_info(opts)
      data
    end

    # Stop Tracking a Package
    # Unsubscribe from tracking updates for a package.
    # @param [Hash] opts the optional parameters
    # @option opts [String] :carrier_code Carrier code used to retrieve tracking information
    # @option opts [String] :tracking_number The tracking number associated with a shipment
    # @return [Array<(String, Integer, Hash)>] String data, response status code and response headers
    def stop_tracking_with_http_info(opts = {})
      if @api_client.config.debugging
        @api_client.config.logger.debug 'Calling API: TrackingApi.stop_tracking ...'
      end
      if @api_client.config.client_side_validation && !opts[:'carrier_code'].nil? && opts[:'carrier_code'].to_s.length < 1
        fail ArgumentError, 'invalid value for "opts[:"carrier_code"]" when calling TrackingApi.stop_tracking, the character length must be great than or equal to 1.'
      end

      if @api_client.config.client_side_validation && !opts[:'tracking_number'].nil? && opts[:'tracking_number'].to_s.length < 1
        fail ArgumentError, 'invalid value for "opts[:"tracking_number"]" when calling TrackingApi.stop_tracking, the character length must be great than or equal to 1.'
      end

      # resource path
      local_var_path = '/v1/tracking/stop'

      # query parameters
      query_params = opts[:query_params] || {}
      query_params[:'carrier_code'] = opts[:'carrier_code'] if !opts[:'carrier_code'].nil?
      query_params[:'tracking_number'] = opts[:'tracking_number'] if !opts[:'tracking_number'].nil?

      # header parameters
      header_params = opts[:header_params] || {}
      # HTTP header 'Accept' (if needed)
      header_params['Accept'] = @api_client.select_header_accept(['text/plain', 'application/json'])

      # form parameters
      form_params = opts[:form_params] || {}

      # http body (model)
      post_body = opts[:body] 

      # return_type
      return_type = opts[:return_type] || 'String' 

      # auth_names
      auth_names = opts[:auth_names] || ['api_key']

      new_options = opts.merge(
        :header_params => header_params,
        :query_params => query_params,
        :form_params => form_params,
        :body => post_body,
        :auth_names => auth_names,
        :return_type => return_type
      )

      data, status_code, headers = @api_client.call_api(:POST, local_var_path, new_options)
      if @api_client.config.debugging
        @api_client.config.logger.debug "API called: TrackingApi#stop_tracking\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
      end
      return data, status_code, headers
    end
  end
end
