=begin
#ShipEngine API

#ShipEngine's easy-to-use REST API lets you manage all of your shipping needs without worrying about the complexities of different carrier APIs and protocols. We handle all the heavy lifting so you can focus on providing a first-class shipping experience for your customers at the best possible prices.  Each of ShipEngine's features can be used by itself or in conjunction with each other to build powerful shipping functionality into your application or service.  ## Getting Started If you're new to REST APIs then be sure to read our [introduction to REST](https://www.shipengine.com/docs/rest/) to understand the basics. Learn how to [authenticate yourself to ShipEngine](https://www.shipengine.com/docs/auth/), and then use our [sandbox environment](https://www.shipengine.com/docs/sandbox/) to kick the tires and get familiar with our API. If you run into any problems, then be sure to check the [error handling guide](https://www.shipengine.com/docs/errors/) for tips.  Here are some step-by-step **tutorials** to get you started:    - [Learn how to create your first shipping label](https://www.shipengine.com/docs/labels/create-a-label/)   - [Calculate shipping costs and compare rates across carriers](https://www.shipengine.com/docs/rates/)   - [Track packages on-demand or in real time](https://www.shipengine.com/docs/tracking/)   - [Validate mailing addresses anywhere on Earth](https://www.shipengine.com/docs/addresses/validation/)   ## Shipping Labels for Every Major Carrier ShipEngine makes it easy to [create shipping labels for any carrier](https://www.shipengine.com/docs/labels/create-a-label/) and [download them](https://www.shipengine.com/docs/labels/downloading/) in a [variety of file formats](https://www.shipengine.com/docs/labels/formats/). You can even customize labels with your own [messages](https://www.shipengine.com/docs/labels/messages/) and [images](https://www.shipengine.com/docs/labels/branding/).   ## Real-Time Package Tracking With ShipEngine you can [get the current status of a package](https://www.shipengine.com/docs/tracking/) or [subscribe to real-time tracking updates](https://www.shipengine.com/docs/tracking/webhooks/) via webhooks. You can also create [custimized tracking pages](https://www.shipengine.com/docs/tracking/branded-tracking-page/) with your own branding so your customers will always know where their package is.   ## Compare Shipping Costs Across Carriers Make sure you ship as cost-effectively as possible by [comparing rates across carriers](https://www.shipengine.com/docs/rates/get-shipment-rates/) using the ShipEngine Rates API. Or if you don't know the full shipment details yet, then you can [get rate estimates](https://www.shipengine.com/docs/rates/estimate/) with limited address info.   ## Worldwide Address Validation ShipEngine supports [address validation](https://www.shipengine.com/docs/addresses/validation/) for virtually [every country on Earth](https://www.shipengine.com/docs/addresses/validation/countries/), including the United States, Canada, Great Britain, Australia, Germany, France, Norway, Spain, Sweden, Israel, Italy, and over 160 others. 

The version of the OpenAPI document: 1.1.202006302006
Contact: sales@shipengine.com
Generated by: https://openapi-generator.tech
OpenAPI Generator version: 4.3.1

=end

require 'cgi'

module OpenapiClient
  class WebhooksApi
    attr_accessor :api_client

    def initialize(api_client = ApiClient.default)
      @api_client = api_client
    end
    # Create a Webhook
    # Create a webook for specific events in the environment.
    # @param create_webhook_request_body [CreateWebhookRequestBody] 
    # @param [Hash] opts the optional parameters
    # @return [CreateWebhookResponseBody]
    def create_webhook(create_webhook_request_body, opts = {})
      data, _status_code, _headers = create_webhook_with_http_info(create_webhook_request_body, opts)
      data
    end

    # Create a Webhook
    # Create a webook for specific events in the environment.
    # @param create_webhook_request_body [CreateWebhookRequestBody] 
    # @param [Hash] opts the optional parameters
    # @return [Array<(CreateWebhookResponseBody, Integer, Hash)>] CreateWebhookResponseBody data, response status code and response headers
    def create_webhook_with_http_info(create_webhook_request_body, opts = {})
      if @api_client.config.debugging
        @api_client.config.logger.debug 'Calling API: WebhooksApi.create_webhook ...'
      end
      # verify the required parameter 'create_webhook_request_body' is set
      if @api_client.config.client_side_validation && create_webhook_request_body.nil?
        fail ArgumentError, "Missing the required parameter 'create_webhook_request_body' when calling WebhooksApi.create_webhook"
      end
      # resource path
      local_var_path = '/v1/environment/webhooks'

      # query parameters
      query_params = opts[:query_params] || {}

      # header parameters
      header_params = opts[:header_params] || {}
      # HTTP header 'Accept' (if needed)
      header_params['Accept'] = @api_client.select_header_accept(['application/json'])
      # HTTP header 'Content-Type'
      header_params['Content-Type'] = @api_client.select_header_content_type(['application/json'])

      # form parameters
      form_params = opts[:form_params] || {}

      # http body (model)
      post_body = opts[:body] || @api_client.object_to_http_body(create_webhook_request_body) 

      # return_type
      return_type = opts[:return_type] || 'CreateWebhookResponseBody' 

      # auth_names
      auth_names = opts[:auth_names] || ['api_key']

      new_options = opts.merge(
        :header_params => header_params,
        :query_params => query_params,
        :form_params => form_params,
        :body => post_body,
        :auth_names => auth_names,
        :return_type => return_type
      )

      data, status_code, headers = @api_client.call_api(:POST, local_var_path, new_options)
      if @api_client.config.debugging
        @api_client.config.logger.debug "API called: WebhooksApi#create_webhook\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
      end
      return data, status_code, headers
    end

    # Delete Webhook By ID
    # Delete a webhook
    # @param webhook_id [String] Webhook ID
    # @param [Hash] opts the optional parameters
    # @return [String]
    def delete_webhook(webhook_id, opts = {})
      data, _status_code, _headers = delete_webhook_with_http_info(webhook_id, opts)
      data
    end

    # Delete Webhook By ID
    # Delete a webhook
    # @param webhook_id [String] Webhook ID
    # @param [Hash] opts the optional parameters
    # @return [Array<(String, Integer, Hash)>] String data, response status code and response headers
    def delete_webhook_with_http_info(webhook_id, opts = {})
      if @api_client.config.debugging
        @api_client.config.logger.debug 'Calling API: WebhooksApi.delete_webhook ...'
      end
      # verify the required parameter 'webhook_id' is set
      if @api_client.config.client_side_validation && webhook_id.nil?
        fail ArgumentError, "Missing the required parameter 'webhook_id' when calling WebhooksApi.delete_webhook"
      end
      if @api_client.config.client_side_validation && webhook_id.to_s.length > 25
        fail ArgumentError, 'invalid value for "webhook_id" when calling WebhooksApi.delete_webhook, the character length must be smaller than or equal to 25.'
      end

      if @api_client.config.client_side_validation && webhook_id.to_s.length < 1
        fail ArgumentError, 'invalid value for "webhook_id" when calling WebhooksApi.delete_webhook, the character length must be great than or equal to 1.'
      end

      pattern = Regexp.new(/^se(-[a-z0-9]+)+$/)
      if @api_client.config.client_side_validation && webhook_id !~ pattern
        fail ArgumentError, "invalid value for 'webhook_id' when calling WebhooksApi.delete_webhook, must conform to the pattern #{pattern}."
      end

      # resource path
      local_var_path = '/v1/environment/webhooks/{webhook_id}'.sub('{' + 'webhook_id' + '}', CGI.escape(webhook_id.to_s))

      # query parameters
      query_params = opts[:query_params] || {}

      # header parameters
      header_params = opts[:header_params] || {}
      # HTTP header 'Accept' (if needed)
      header_params['Accept'] = @api_client.select_header_accept(['text/plain', 'application/json'])

      # form parameters
      form_params = opts[:form_params] || {}

      # http body (model)
      post_body = opts[:body] 

      # return_type
      return_type = opts[:return_type] || 'String' 

      # auth_names
      auth_names = opts[:auth_names] || ['api_key']

      new_options = opts.merge(
        :header_params => header_params,
        :query_params => query_params,
        :form_params => form_params,
        :body => post_body,
        :auth_names => auth_names,
        :return_type => return_type
      )

      data, status_code, headers = @api_client.call_api(:DELETE, local_var_path, new_options)
      if @api_client.config.debugging
        @api_client.config.logger.debug "API called: WebhooksApi#delete_webhook\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
      end
      return data, status_code, headers
    end

    # Get Webhook By ID
    # Retrieve individual webhook by an ID
    # @param webhook_id [String] Webhook ID
    # @param [Hash] opts the optional parameters
    # @return [GetWebhookByIdResponseBody]
    def get_webhook_by_id(webhook_id, opts = {})
      data, _status_code, _headers = get_webhook_by_id_with_http_info(webhook_id, opts)
      data
    end

    # Get Webhook By ID
    # Retrieve individual webhook by an ID
    # @param webhook_id [String] Webhook ID
    # @param [Hash] opts the optional parameters
    # @return [Array<(GetWebhookByIdResponseBody, Integer, Hash)>] GetWebhookByIdResponseBody data, response status code and response headers
    def get_webhook_by_id_with_http_info(webhook_id, opts = {})
      if @api_client.config.debugging
        @api_client.config.logger.debug 'Calling API: WebhooksApi.get_webhook_by_id ...'
      end
      # verify the required parameter 'webhook_id' is set
      if @api_client.config.client_side_validation && webhook_id.nil?
        fail ArgumentError, "Missing the required parameter 'webhook_id' when calling WebhooksApi.get_webhook_by_id"
      end
      if @api_client.config.client_side_validation && webhook_id.to_s.length > 25
        fail ArgumentError, 'invalid value for "webhook_id" when calling WebhooksApi.get_webhook_by_id, the character length must be smaller than or equal to 25.'
      end

      if @api_client.config.client_side_validation && webhook_id.to_s.length < 1
        fail ArgumentError, 'invalid value for "webhook_id" when calling WebhooksApi.get_webhook_by_id, the character length must be great than or equal to 1.'
      end

      pattern = Regexp.new(/^se(-[a-z0-9]+)+$/)
      if @api_client.config.client_side_validation && webhook_id !~ pattern
        fail ArgumentError, "invalid value for 'webhook_id' when calling WebhooksApi.get_webhook_by_id, must conform to the pattern #{pattern}."
      end

      # resource path
      local_var_path = '/v1/environment/webhooks/{webhook_id}'.sub('{' + 'webhook_id' + '}', CGI.escape(webhook_id.to_s))

      # query parameters
      query_params = opts[:query_params] || {}

      # header parameters
      header_params = opts[:header_params] || {}
      # HTTP header 'Accept' (if needed)
      header_params['Accept'] = @api_client.select_header_accept(['application/json'])

      # form parameters
      form_params = opts[:form_params] || {}

      # http body (model)
      post_body = opts[:body] 

      # return_type
      return_type = opts[:return_type] || 'GetWebhookByIdResponseBody' 

      # auth_names
      auth_names = opts[:auth_names] || ['api_key']

      new_options = opts.merge(
        :header_params => header_params,
        :query_params => query_params,
        :form_params => form_params,
        :body => post_body,
        :auth_names => auth_names,
        :return_type => return_type
      )

      data, status_code, headers = @api_client.call_api(:GET, local_var_path, new_options)
      if @api_client.config.debugging
        @api_client.config.logger.debug "API called: WebhooksApi#get_webhook_by_id\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
      end
      return data, status_code, headers
    end

    # List Webhooks
    # List all webhooks currently enabled for the account.
    # @param [Hash] opts the optional parameters
    # @return [Array<Webhook>]
    def list_webhooks(opts = {})
      data, _status_code, _headers = list_webhooks_with_http_info(opts)
      data
    end

    # List Webhooks
    # List all webhooks currently enabled for the account.
    # @param [Hash] opts the optional parameters
    # @return [Array<(Array<Webhook>, Integer, Hash)>] Array<Webhook> data, response status code and response headers
    def list_webhooks_with_http_info(opts = {})
      if @api_client.config.debugging
        @api_client.config.logger.debug 'Calling API: WebhooksApi.list_webhooks ...'
      end
      # resource path
      local_var_path = '/v1/environment/webhooks'

      # query parameters
      query_params = opts[:query_params] || {}

      # header parameters
      header_params = opts[:header_params] || {}
      # HTTP header 'Accept' (if needed)
      header_params['Accept'] = @api_client.select_header_accept(['application/json'])

      # form parameters
      form_params = opts[:form_params] || {}

      # http body (model)
      post_body = opts[:body] 

      # return_type
      return_type = opts[:return_type] || 'Array<Webhook>' 

      # auth_names
      auth_names = opts[:auth_names] || ['api_key']

      new_options = opts.merge(
        :header_params => header_params,
        :query_params => query_params,
        :form_params => form_params,
        :body => post_body,
        :auth_names => auth_names,
        :return_type => return_type
      )

      data, status_code, headers = @api_client.call_api(:GET, local_var_path, new_options)
      if @api_client.config.debugging
        @api_client.config.logger.debug "API called: WebhooksApi#list_webhooks\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
      end
      return data, status_code, headers
    end

    # Update a Webhook
    # Update the webhook url property
    # @param webhook_id [String] Webhook ID
    # @param update_webhook_request_body [UpdateWebhookRequestBody] 
    # @param [Hash] opts the optional parameters
    # @return [String]
    def update_webhook(webhook_id, update_webhook_request_body, opts = {})
      data, _status_code, _headers = update_webhook_with_http_info(webhook_id, update_webhook_request_body, opts)
      data
    end

    # Update a Webhook
    # Update the webhook url property
    # @param webhook_id [String] Webhook ID
    # @param update_webhook_request_body [UpdateWebhookRequestBody] 
    # @param [Hash] opts the optional parameters
    # @return [Array<(String, Integer, Hash)>] String data, response status code and response headers
    def update_webhook_with_http_info(webhook_id, update_webhook_request_body, opts = {})
      if @api_client.config.debugging
        @api_client.config.logger.debug 'Calling API: WebhooksApi.update_webhook ...'
      end
      # verify the required parameter 'webhook_id' is set
      if @api_client.config.client_side_validation && webhook_id.nil?
        fail ArgumentError, "Missing the required parameter 'webhook_id' when calling WebhooksApi.update_webhook"
      end
      if @api_client.config.client_side_validation && webhook_id.to_s.length > 25
        fail ArgumentError, 'invalid value for "webhook_id" when calling WebhooksApi.update_webhook, the character length must be smaller than or equal to 25.'
      end

      if @api_client.config.client_side_validation && webhook_id.to_s.length < 1
        fail ArgumentError, 'invalid value for "webhook_id" when calling WebhooksApi.update_webhook, the character length must be great than or equal to 1.'
      end

      pattern = Regexp.new(/^se(-[a-z0-9]+)+$/)
      if @api_client.config.client_side_validation && webhook_id !~ pattern
        fail ArgumentError, "invalid value for 'webhook_id' when calling WebhooksApi.update_webhook, must conform to the pattern #{pattern}."
      end

      # verify the required parameter 'update_webhook_request_body' is set
      if @api_client.config.client_side_validation && update_webhook_request_body.nil?
        fail ArgumentError, "Missing the required parameter 'update_webhook_request_body' when calling WebhooksApi.update_webhook"
      end
      # resource path
      local_var_path = '/v1/environment/webhooks/{webhook_id}'.sub('{' + 'webhook_id' + '}', CGI.escape(webhook_id.to_s))

      # query parameters
      query_params = opts[:query_params] || {}

      # header parameters
      header_params = opts[:header_params] || {}
      # HTTP header 'Accept' (if needed)
      header_params['Accept'] = @api_client.select_header_accept(['text/plain', 'application/json'])
      # HTTP header 'Content-Type'
      header_params['Content-Type'] = @api_client.select_header_content_type(['application/json'])

      # form parameters
      form_params = opts[:form_params] || {}

      # http body (model)
      post_body = opts[:body] || @api_client.object_to_http_body(update_webhook_request_body) 

      # return_type
      return_type = opts[:return_type] || 'String' 

      # auth_names
      auth_names = opts[:auth_names] || ['api_key']

      new_options = opts.merge(
        :header_params => header_params,
        :query_params => query_params,
        :form_params => form_params,
        :body => post_body,
        :auth_names => auth_names,
        :return_type => return_type
      )

      data, status_code, headers = @api_client.call_api(:PUT, local_var_path, new_options)
      if @api_client.config.debugging
        @api_client.config.logger.debug "API called: WebhooksApi#update_webhook\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
      end
      return data, status_code, headers
    end
  end
end
