=begin
#ShipEngine API

#ShipEngine's easy-to-use REST API lets you manage all of your shipping needs without worrying about the complexities of different carrier APIs and protocols. We handle all the heavy lifting so you can focus on providing a first-class shipping experience for your customers at the best possible prices.  Each of ShipEngine's features can be used by itself or in conjunction with each other to build powerful shipping functionality into your application or service.  ## Getting Started If you're new to REST APIs then be sure to read our [introduction to REST](https://www.shipengine.com/docs/rest/) to understand the basics. Learn how to [authenticate yourself to ShipEngine](https://www.shipengine.com/docs/auth/), and then use our [sandbox environment](https://www.shipengine.com/docs/sandbox/) to kick the tires and get familiar with our API. If you run into any problems, then be sure to check the [error handling guide](https://www.shipengine.com/docs/errors/) for tips.  Here are some step-by-step **tutorials** to get you started:    - [Learn how to create your first shipping label](https://www.shipengine.com/docs/labels/create-a-label/)   - [Calculate shipping costs and compare rates across carriers](https://www.shipengine.com/docs/rates/)   - [Track packages on-demand or in real time](https://www.shipengine.com/docs/tracking/)   - [Validate mailing addresses anywhere on Earth](https://www.shipengine.com/docs/addresses/validation/)   ## Shipping Labels for Every Major Carrier ShipEngine makes it easy to [create shipping labels for any carrier](https://www.shipengine.com/docs/labels/create-a-label/) and [download them](https://www.shipengine.com/docs/labels/downloading/) in a [variety of file formats](https://www.shipengine.com/docs/labels/formats/). You can even customize labels with your own [messages](https://www.shipengine.com/docs/labels/messages/) and [images](https://www.shipengine.com/docs/labels/branding/).   ## Real-Time Package Tracking With ShipEngine you can [get the current status of a package](https://www.shipengine.com/docs/tracking/) or [subscribe to real-time tracking updates](https://www.shipengine.com/docs/tracking/webhooks/) via webhooks. You can also create [custimized tracking pages](https://www.shipengine.com/docs/tracking/branded-tracking-page/) with your own branding so your customers will always know where their package is.   ## Compare Shipping Costs Across Carriers Make sure you ship as cost-effectively as possible by [comparing rates across carriers](https://www.shipengine.com/docs/rates/get-shipment-rates/) using the ShipEngine Rates API. Or if you don't know the full shipment details yet, then you can [get rate estimates](https://www.shipengine.com/docs/rates/estimate/) with limited address info.   ## Worldwide Address Validation ShipEngine supports [address validation](https://www.shipengine.com/docs/addresses/validation/) for virtually [every country on Earth](https://www.shipengine.com/docs/addresses/validation/countries/), including the United States, Canada, Great Britain, Australia, Germany, France, Norway, Spain, Sweden, Israel, Italy, and over 160 others. 

The version of the OpenAPI document: 1.1.202006302006
Contact: sales@shipengine.com
Generated by: https://openapi-generator.tech
OpenAPI Generator version: 4.3.1

=end

require 'date'

module OpenapiClient
  # Advanced shipment options
  class AdvancedShipmentOptions
    # This field is used to [bill shipping costs to a third party](https://www.shipengine.com/docs/shipping/bill-to-third-party/). This field must be used in conjunction with the `bill_to_country_code`, `bill_to_party`, and `bill_to_postal_code` fields. 
    attr_accessor :bill_to_account

    # The two-letter [ISO 3166-1 country code](https://en.wikipedia.org/wiki/ISO_3166-1) of the third-party that is responsible for shipping costs. 
    attr_accessor :bill_to_country_code

    # Indicates whether to bill shipping costs to the recipient or to a third-party.  When billing to a third-party, the `bill_to_account`, `bill_to_country_code`, and `bill_to_postal_code` fields must also be set. 
    attr_accessor :bill_to_party

    # The postal code of the third-party that is responsible for shipping costs. 
    attr_accessor :bill_to_postal_code

    # Indicates that the shipment contains alcohol.
    attr_accessor :contains_alcohol

    # Indicates that the shipper is paying the international delivery duties for this shipment.  This option is supported by UPS, FedEx, and DHL Express. 
    attr_accessor :delivered_duty_paid

    # Indicates if the shipment contain dry ice
    attr_accessor :dry_ice

    # The weight of the dry ice in the shipment
    attr_accessor :dry_ice_weight

    # Indicates that the package cannot be processed automatically because it is too large or irregularly shaped. This is primarily for USPS shipments.  See [Section 1.2 of the USPS parcel standards](https://pe.usps.com/text/dmm300/101.htm#ep1047495) for details. 
    attr_accessor :non_machinable

    # Enables Saturday delivery, if supported by the carrier.
    attr_accessor :saturday_delivery

    # Whether to use [UPS Ground Freight pricing](https://www.shipengine.com/docs/shipping/ups-ground-freight/). If enabled, then a `freight_class` must also be specified. 
    attr_accessor :use_ups_ground_freight_pricing

    # The National Motor Freight Traffic Association [freight class](http://www.nmfta.org/pages/nmfc?AspxAutoDetectCookieSupport=1), such as \"77.5\", \"110\", or \"250\". 
    attr_accessor :freight_class

    # An arbitrary field that can be used to store information about the shipment. 
    attr_accessor :custom_field1

    # An arbitrary field that can be used to store information about the shipment. 
    attr_accessor :custom_field2

    # An arbitrary field that can be used to store information about the shipment. 
    attr_accessor :custom_field3

    attr_accessor :collect_on_delivery

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :'bill_to_account' => :'bill_to_account',
        :'bill_to_country_code' => :'bill_to_country_code',
        :'bill_to_party' => :'bill_to_party',
        :'bill_to_postal_code' => :'bill_to_postal_code',
        :'contains_alcohol' => :'contains_alcohol',
        :'delivered_duty_paid' => :'delivered_duty_paid',
        :'dry_ice' => :'dry_ice',
        :'dry_ice_weight' => :'dry_ice_weight',
        :'non_machinable' => :'non_machinable',
        :'saturday_delivery' => :'saturday_delivery',
        :'use_ups_ground_freight_pricing' => :'use_ups_ground_freight_pricing',
        :'freight_class' => :'freight_class',
        :'custom_field1' => :'custom_field1',
        :'custom_field2' => :'custom_field2',
        :'custom_field3' => :'custom_field3',
        :'collect_on_delivery' => :'collect_on_delivery'
      }
    end

    # Attribute type mapping.
    def self.openapi_types
      {
        :'bill_to_account' => :'String',
        :'bill_to_country_code' => :'String',
        :'bill_to_party' => :'BillToParty',
        :'bill_to_postal_code' => :'String',
        :'contains_alcohol' => :'Boolean',
        :'delivered_duty_paid' => :'Boolean',
        :'dry_ice' => :'Boolean',
        :'dry_ice_weight' => :'Weight',
        :'non_machinable' => :'Boolean',
        :'saturday_delivery' => :'Boolean',
        :'use_ups_ground_freight_pricing' => :'Boolean',
        :'freight_class' => :'String',
        :'custom_field1' => :'String',
        :'custom_field2' => :'String',
        :'custom_field3' => :'String',
        :'collect_on_delivery' => :'CollectOnDelivery'
      }
    end

    # List of attributes with nullable: true
    def self.openapi_nullable
      Set.new([
        :'bill_to_account',
        :'bill_to_country_code',
        :'bill_to_party',
        :'bill_to_postal_code',
        :'dry_ice_weight',
        :'use_ups_ground_freight_pricing',
        :'freight_class',
        :'custom_field1',
        :'custom_field2',
        :'custom_field3',
      ])
    end

    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    def initialize(attributes = {})
      if (!attributes.is_a?(Hash))
        fail ArgumentError, "The input argument (attributes) must be a hash in `OpenapiClient::AdvancedShipmentOptions` initialize method"
      end

      # check to see if the attribute exists and convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h|
        if (!self.class.attribute_map.key?(k.to_sym))
          fail ArgumentError, "`#{k}` is not a valid attribute in `OpenapiClient::AdvancedShipmentOptions`. Please check the name to make sure it's valid. List of attributes: " + self.class.attribute_map.keys.inspect
        end
        h[k.to_sym] = v
      }

      if attributes.key?(:'bill_to_account')
        self.bill_to_account = attributes[:'bill_to_account']
      else
        self.bill_to_account = 'null'
      end

      if attributes.key?(:'bill_to_country_code')
        self.bill_to_country_code = attributes[:'bill_to_country_code']
      end

      if attributes.key?(:'bill_to_party')
        self.bill_to_party = attributes[:'bill_to_party']
      end

      if attributes.key?(:'bill_to_postal_code')
        self.bill_to_postal_code = attributes[:'bill_to_postal_code']
      else
        self.bill_to_postal_code = 'null'
      end

      if attributes.key?(:'contains_alcohol')
        self.contains_alcohol = attributes[:'contains_alcohol']
      else
        self.contains_alcohol = false
      end

      if attributes.key?(:'delivered_duty_paid')
        self.delivered_duty_paid = attributes[:'delivered_duty_paid']
      else
        self.delivered_duty_paid = false
      end

      if attributes.key?(:'dry_ice')
        self.dry_ice = attributes[:'dry_ice']
      else
        self.dry_ice = false
      end

      if attributes.key?(:'dry_ice_weight')
        self.dry_ice_weight = attributes[:'dry_ice_weight']
      end

      if attributes.key?(:'non_machinable')
        self.non_machinable = attributes[:'non_machinable']
      else
        self.non_machinable = false
      end

      if attributes.key?(:'saturday_delivery')
        self.saturday_delivery = attributes[:'saturday_delivery']
      else
        self.saturday_delivery = false
      end

      if attributes.key?(:'use_ups_ground_freight_pricing')
        self.use_ups_ground_freight_pricing = attributes[:'use_ups_ground_freight_pricing']
      end

      if attributes.key?(:'freight_class')
        self.freight_class = attributes[:'freight_class']
      else
        self.freight_class = 'null'
      end

      if attributes.key?(:'custom_field1')
        self.custom_field1 = attributes[:'custom_field1']
      else
        self.custom_field1 = 'null'
      end

      if attributes.key?(:'custom_field2')
        self.custom_field2 = attributes[:'custom_field2']
      else
        self.custom_field2 = 'null'
      end

      if attributes.key?(:'custom_field3')
        self.custom_field3 = attributes[:'custom_field3']
      else
        self.custom_field3 = 'null'
      end

      if attributes.key?(:'collect_on_delivery')
        self.collect_on_delivery = attributes[:'collect_on_delivery']
      end
    end

    # Show invalid properties with the reasons. Usually used together with valid?
    # @return Array for valid properties with the reasons
    def list_invalid_properties
      invalid_properties = Array.new
      if !@custom_field1.nil? && @custom_field1.to_s.length > 100
        invalid_properties.push('invalid value for "custom_field1", the character length must be smaller than or equal to 100.')
      end

      if !@custom_field2.nil? && @custom_field2.to_s.length > 100
        invalid_properties.push('invalid value for "custom_field2", the character length must be smaller than or equal to 100.')
      end

      if !@custom_field3.nil? && @custom_field3.to_s.length > 100
        invalid_properties.push('invalid value for "custom_field3", the character length must be smaller than or equal to 100.')
      end

      invalid_properties
    end

    # Check to see if the all the properties in the model are valid
    # @return true if the model is valid
    def valid?
      return false if !@custom_field1.nil? && @custom_field1.to_s.length > 100
      return false if !@custom_field2.nil? && @custom_field2.to_s.length > 100
      return false if !@custom_field3.nil? && @custom_field3.to_s.length > 100
      true
    end

    # Custom attribute writer method with validation
    # @param [Object] custom_field1 Value to be assigned
    def custom_field1=(custom_field1)
      if !custom_field1.nil? && custom_field1.to_s.length > 100
        fail ArgumentError, 'invalid value for "custom_field1", the character length must be smaller than or equal to 100.'
      end

      @custom_field1 = custom_field1
    end

    # Custom attribute writer method with validation
    # @param [Object] custom_field2 Value to be assigned
    def custom_field2=(custom_field2)
      if !custom_field2.nil? && custom_field2.to_s.length > 100
        fail ArgumentError, 'invalid value for "custom_field2", the character length must be smaller than or equal to 100.'
      end

      @custom_field2 = custom_field2
    end

    # Custom attribute writer method with validation
    # @param [Object] custom_field3 Value to be assigned
    def custom_field3=(custom_field3)
      if !custom_field3.nil? && custom_field3.to_s.length > 100
        fail ArgumentError, 'invalid value for "custom_field3", the character length must be smaller than or equal to 100.'
      end

      @custom_field3 = custom_field3
    end

    # Checks equality by comparing each attribute.
    # @param [Object] Object to be compared
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          bill_to_account == o.bill_to_account &&
          bill_to_country_code == o.bill_to_country_code &&
          bill_to_party == o.bill_to_party &&
          bill_to_postal_code == o.bill_to_postal_code &&
          contains_alcohol == o.contains_alcohol &&
          delivered_duty_paid == o.delivered_duty_paid &&
          dry_ice == o.dry_ice &&
          dry_ice_weight == o.dry_ice_weight &&
          non_machinable == o.non_machinable &&
          saturday_delivery == o.saturday_delivery &&
          use_ups_ground_freight_pricing == o.use_ups_ground_freight_pricing &&
          freight_class == o.freight_class &&
          custom_field1 == o.custom_field1 &&
          custom_field2 == o.custom_field2 &&
          custom_field3 == o.custom_field3 &&
          collect_on_delivery == o.collect_on_delivery
    end

    # @see the `==` method
    # @param [Object] Object to be compared
    def eql?(o)
      self == o
    end

    # Calculates hash code according to all attributes.
    # @return [Integer] Hash code
    def hash
      [bill_to_account, bill_to_country_code, bill_to_party, bill_to_postal_code, contains_alcohol, delivered_duty_paid, dry_ice, dry_ice_weight, non_machinable, saturday_delivery, use_ups_ground_freight_pricing, freight_class, custom_field1, custom_field2, custom_field3, collect_on_delivery].hash
    end

    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def self.build_from_hash(attributes)
      new.build_from_hash(attributes)
    end

    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.openapi_types.each_pair do |key, type|
        if type =~ /\AArray<(.*)>/i
          # check to ensure the input is an array given that the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map { |v| _deserialize($1, v) })
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        end # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end

    # Deserializes the data based on type
    # @param string type Data type
    # @param string value Value to be deserialized
    # @return [Object] Deserialized data
    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :Boolean
        if value.to_s =~ /\A(true|t|yes|y|1)\z/i
          true
        else
          false
        end
      when :Object
        # generic object (usually a Hash), return directly
        value
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+?), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      else # model
        OpenapiClient.const_get(type).build_from_hash(value)
      end
    end

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_hash (backward compatibility)
    # @return [Hash] Returns the object in the form of hash
    def to_body
      to_hash
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        if value.nil?
          is_nullable = self.class.openapi_nullable.include?(attr)
          next if !is_nullable || (is_nullable && !instance_variable_defined?(:"@#{attr}"))
        end
        
        hash[param] = _to_hash(value)
      end
      hash
    end

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
